@@ -1,5 +1,10 @@
 {-# LANGUAGE FlexibleContexts #-}
 {-# LANGUAGE CPP #-}
+{-# LANGUAGE ScopedTypeVariables #-}
+
+#ifndef MIN_VERSION_base
+#define MIN_VERSION_base(x,y,z) 1
+#endif
 
 -- | IEEE-754 parsing, as described in this stack-overflow article:
 --
@@ -21,18 +26,17 @@ module Data.Serialize.IEEE754 (
 
 ) where
 
-import Control.Applicative ( (<$>) )
-import Control.Monad.ST ( runST, ST )
-
-import Data.Array.ST ( newArray, readArray, MArray, STUArray )
 import Data.Word ( Word32, Word64 )
 import Data.Serialize.Get
 import Data.Serialize.Put
+import qualified Data.ByteString.Builder as Builder
+import System.IO.Unsafe (unsafeDupablePerformIO)
+import Foreign.Marshal.Alloc (alloca)
+import Foreign.Storable (peek, poke)
+import Foreign.Ptr (castPtr, Ptr)
 
-#if __GLASGOW_HASKELL__ >= 704
-import Data.Array.Unsafe (castSTUArray)
-#else
-import Data.Array.ST (castSTUArray)
+#if !(MIN_VERSION_base(4,8,0))
+import Control.Applicative ( (<$>) )
 #endif
 
 -- | Read a Float in little endian IEEE-754 format
@@ -53,38 +57,28 @@ getFloat64be = wordToDouble <$> getWord64be
 
 -- | Write a Float in little endian IEEE-754 format
 putFloat32le :: Float -> Put
-putFloat32le = putWord32le . floatToWord
+putFloat32le = putBuilder . Builder.floatLE
 
 -- | Write a Float in big endian IEEE-754 format
 putFloat32be :: Float -> Put
-putFloat32be = putWord32be . floatToWord
+putFloat32be = putBuilder . Builder.floatBE
 
 -- | Write a Double in little endian IEEE-754 format
 putFloat64le :: Double -> Put
-putFloat64le = putWord64le . doubleToWord
+putFloat64le = putBuilder . Builder.doubleLE
 
 -- | Write a Double in big endian IEEE-754 format
 putFloat64be :: Double -> Put
-putFloat64be = putWord64be . doubleToWord
+putFloat64be = putBuilder . Builder.doubleBE
 
 {-# INLINE wordToFloat #-}
 wordToFloat :: Word32 -> Float
-wordToFloat x = runST (cast x)
-
-{-# INLINE floatToWord #-}
-floatToWord :: Float -> Word32
-floatToWord x = runST (cast x)
+wordToFloat w = unsafeDupablePerformIO $ alloca $ \(ptr :: Ptr Word32) -> do
+    poke ptr w
+    peek (castPtr ptr)
 
 {-# INLINE wordToDouble #-}
 wordToDouble :: Word64 -> Double
-wordToDouble x = runST (cast x)
-
-{-# INLINE doubleToWord #-}
-doubleToWord :: Double -> Word64
-doubleToWord x = runST (cast x)
-
-{-# INLINE cast #-}
-cast :: (MArray (STUArray s) a (ST s),
-         MArray (STUArray s) b (ST s)) =>
-        a -> ST s b
-cast x = newArray (0 :: Int, 0) x >>= castSTUArray >>= flip readArray 0
+wordToDouble w = unsafeDupablePerformIO $ alloca $ \(ptr :: Ptr Word64) -> do
+    poke ptr w
+    peek (castPtr ptr)
